"""
Custom django-crispy-forms layouts.
"""
from __future__ import unicode_literals
from django.utils.translation import ugettext_lazy as _
from crispy_forms import layout
from crispy_forms.helper import FormHelper
from crispy_forms.utils import flatatt
from django.template.loader import render_to_string
from django_cradmin.templatetags.cradmin_icon_tags import cradmin_icon


class CradminSubmitButton(layout.Submit):
    template = 'django_cradmin/crispylayouts/submitbutton.django.html'
    extra_button_attributes = {}

    def get_extra_button_attributes(self):
        return self.extra_button_attributes

    def __init__(self, name, value, icon_cssclass=None, **kwargs):
        self.extra_button_attributes = flatatt(self.get_extra_button_attributes())
        self.icon_cssclass = icon_cssclass
        super(CradminSubmitButton, self).__init__(name, value, **kwargs)


class PrimarySubmit(CradminSubmitButton):
    field_classes = "btn btn-primary"


class PrimarySubmitLg(CradminSubmitButton):
    field_classes = "btn btn-primary btn-lg"


class DangerSubmit(CradminSubmitButton):
    field_classes = "btn btn-danger"


class DefaultSubmit(CradminSubmitButton):
    field_classes = "btn btn-default"


class CollapsedSectionLayout(layout.Div):
    wrapper_template_name = 'django_cradmin/crispylayouts/collapsed-section-layout.django.html'

    def __init__(self, *args, **kwargs):
        self.show_label = kwargs.pop('show_label', _('Show'))
        self.hide_label = kwargs.pop('hide_label', _('Hide'))
        self.show_icon = kwargs.pop('show_icon', cradmin_icon('caret-down'))
        self.hide_icon = kwargs.pop('hide_icon', cradmin_icon('caret-up'))
        super(CollapsedSectionLayout, self).__init__(*args, **kwargs)

    def get_wrapper_context_data(self):
        return {
            'show_label': self.show_label,
            'hide_label': self.hide_label,
            'show_icon': self.show_icon,
            'hide_icon': self.hide_icon,
        }

    def render(self, *args, **kwargs):
        content = super(CollapsedSectionLayout, self).render(*args, **kwargs)
        context = {
            'content': content
        }
        context.update(self.get_wrapper_context_data())
        return render_to_string(self.wrapper_template_name, context)


class CradminFormHelper(FormHelper):
    template = 'django_cradmin/crispylayouts/cradmin_form_helper.django.html'



from __future__ import unicode_literals
from django.template import Template, Context, TemplateSyntaxError
from django.test import TestCase


class TestCradminWrapText(TestCase):
    def __render_template(self, template):
        full_template = '{{% load cradmin_textformatting_tags %}}{}'.format(template)
        return Template(full_template).render(Context({}))

    def test_does_not_change_short_lines(self):
        self.assertEqual(
            self.__render_template('{% cradmin_wrap_text 5 %}Test{% end_cradmin_wrap_text %}'),
            'Test')

    def test_wraps_long_lines(self):
        self.assertEqual(
            self.__render_template('{% cradmin_wrap_text 5 %}Test Stuff{% end_cradmin_wrap_text %}'),
            'Test\nStuff')

    def test_does_not_break_long_words(self):
        self.assertEqual(
            self.__render_template('{% cradmin_wrap_text 5 %}TestStuff{% end_cradmin_wrap_text %}'),
            'TestStuff')

    def test_keep_indent(self):
        self.assertEqual(
            self.__render_template('{% cradmin_wrap_text 5 indentspaces=4 %}Test Stuff{% end_cradmin_wrap_text %}'),
            '    Test\n    Stuff')

    def test_no_max_line_length(self):
        with self.assertRaises(TemplateSyntaxError):
            self.__render_template('{% cradmin_wrap_text %}Test{% end_cradmin_wrap_text %}')



from __future__ import unicode_literals

import json

from django import template

from django_cradmin import crapp
from django_cradmin.crinstance import reverse_cradmin_url
from django_cradmin.registry import cradmin_instance_registry

register = template.Library()


@register.simple_tag(takes_context=True)
def cradmin_titletext_for_role(context, role):
    """
    Template tag implementation of
    :meth:`django_cradmin.crinstance.BaseCrAdminInstance.get_titletext_for_role`.
    """
    request = context['request']
    cradmin_instance = cradmin_instance_registry.get_current_instance(request)
    return cradmin_instance.get_titletext_for_role(role)


@register.assignment_tag(takes_context=True)
def cradmin_descriptionhtml_for_role(context, role):
    """
    Template tag implementation of
    :meth:`django_cradmin.crinstance.BaseCrAdminInstance.get_titletext_for_role`.
    """
    request = context['request']
    cradmin_instance = cradmin_instance_registry.get_current_instance(request)
    return cradmin_instance.get_descriptionhtml_for_role(role)


@register.simple_tag(takes_context=True)
def cradmin_rolefrontpage_url(context, role):
    """
    Template tag implementation of
    :meth:`django_cradmin.crinstance.BaseCrAdminInstance.rolefrontpage_url`.
    """
    request = context['request']
    cradmin_instance = cradmin_instance_registry.get_current_instance(request)
    return cradmin_instance.rolefrontpage_url(cradmin_instance.get_roleid(role))


@register.simple_tag(takes_context=True)
def cradmin_appurl(context, viewname, *args, **kwargs):
    """
    Template tag implementation of :meth:`django_cradmin.crapp.App.reverse_appurl`.
    """
    request = context['request']
    return request.cradmin_app.reverse_appurl(viewname, args=args, kwargs=kwargs)


@register.simple_tag(takes_context=True)
def cradmin_instance_url(context, appname, viewname, *args, **kwargs):
    """
    Template tag implementation of :meth:`django_cradmin.crinstance.BaseCrAdminInstance.reverse_url`.
    """
    request = context['request']
    return request.cradmin_instance.reverse_url(
        appname=appname, viewname=viewname, args=args, kwargs=kwargs)


@register.simple_tag(takes_context=True)
def cradmin_instanceindex_url(context, appname):
    """
    Template tag implementation of :meth:`django_cradmin.crinstance.BaseCrAdminInstance.appindex_url`.
    """
    request = context['request']
    return request.cradmin_instance.reverse_url(
        appname=appname, viewname=crapp.INDEXVIEW_NAME)


@register.simple_tag(takes_context=True)
def cradmin_url(context, instanceid, appname, roleid, viewname, *args, **kwargs):
    """
    Template tag implementation of :meth:`django_cradmin.crinstance.reverse_cradmin_url`.
    """
    return reverse_cradmin_url(
        instanceid=instanceid,
        appname=appname,
        roleid=roleid,
        viewname=viewname, args=args, kwargs=kwargs)


@register.simple_tag(takes_context=True)
def cradmin_render_menu(context):
    """
    Template tag that renders the cradmin menu.

    We use this instead of an include tag to handle some issues
    with mocking tests.
    """
    request = context['request']
    if hasattr(request, 'cradmin_instance'):
        return request.cradmin_instance.get_menu().render(context)
    return ''


@register.filter
def cradmin_jsonencode(json_serializable_pythonobject):
    """
    Template filter that converts a json serializable object
    to a json encoded string.
    """
    return json.dumps(json_serializable_pythonobject)



from django.conf import settings


def get_setting(settingvariable, fallback=None):
    return getattr(settings, settingvariable, fallback)


def cradmin(request):
    return {
        'DJANGO_CRADMIN_THEME_PATH': get_setting(
            'DJANGO_CRADMIN_THEME_PATH',
            'django_cradmin/dist/css/cradmin_theme_default/theme.css'),
        'DJANGO_CRADMIN_MENU_SCROLL_TOP_FIXED': get_setting(
            'DJANGO_CRADMIN_MENU_SCROLL_TOP_FIXED', False),
        'DJANGO_CRADMIN_CSS_ICON_LIBRARY_PATH': get_setting(
            'DJANGO_CRADMIN_CSS_ICON_LIBRARY_PATH',
            'django_cradmin/dist/vendor/fonts/fontawesome/css/font-awesome.min.css'),
    }



from __future__ import unicode_literals
from datetime import datetime, timedelta
from django.core.management import call_command
from django.test import TestCase
from django.utils import timezone
import mock
from django_cradmin.apps.cradmin_generic_token_with_metadata.models import GenericTokenWithMetadata, generate_token, \
    get_expiration_datetime_for_app
from django_cradmin.tests.helpers import create_user


class TestGenericTokenWithMetadata(TestCase):
    def _create_generic_token_with_metadata(self, user, created_datetime=None, expiration_datetime=None,
                                            app='testapp', **kwargs):
        generic_token_with_metadata = GenericTokenWithMetadata.objects.create(
            created_datetime=(created_datetime or timezone.now()),
            expiration_datetime=(expiration_datetime or (timezone.now() + timedelta(days=2))),
            app=app, content_object=user, **kwargs)
        return generic_token_with_metadata

    def test_generate_token(self):
        self.assertEqual(len(generate_token()), 73)
        self.assertNotEqual(generate_token(), generate_token())

    def test_generate(self):
        testuser = create_user('testuser')
        unique_user_token = GenericTokenWithMetadata.objects.generate(
            content_object=testuser, app='testapp',
            expiration_datetime=get_expiration_datetime_for_app('testapp'))
        self.assertEqual(unique_user_token.content_object, testuser)
        self.assertEqual(unique_user_token.app, 'testapp')
        self.assertEqual(len(unique_user_token.token), 73)

    def test_generate_handle_not_unique(self):
        self._create_generic_token_with_metadata(user=create_user('testuser1'), app='testapp1', token='taken')
        tokens = iter(['taken', 'free'])
        with mock.patch('django_cradmin.apps.cradmin_generic_token_with_metadata.models.generate_token',
                        lambda: next(tokens)):
            unique_user_token = GenericTokenWithMetadata.objects.generate(
                content_object=create_user('testuser2'), app='testapp2',
                expiration_datetime=get_expiration_datetime_for_app('testapp'))
        self.assertEqual(unique_user_token.token, 'free')

    def test_filter_by_content_object(self):
        testobject1 = create_user('testobject1')
        testobject2 = create_user('testobject2')
        generictoken = GenericTokenWithMetadata.objects.generate(
            content_object=testobject1, app='testapp',
            expiration_datetime=get_expiration_datetime_for_app('testapp'))
        self.assertEquals(
            GenericTokenWithMetadata.objects.filter_by_content_object(testobject1).get(),
            generictoken)
        self.assertEquals(GenericTokenWithMetadata.objects.filter_by_content_object(testobject2).count(), 0)

    def test_unsafe_pop(self):
        testuser = create_user('testuser')
        self._create_generic_token_with_metadata(user=testuser, app='testapp1', token='test-token1')
        self._create_generic_token_with_metadata(user=testuser, app='testapp2', token='test-token2')
        self.assertEquals(GenericTokenWithMetadata.objects.unsafe_pop(
            token='test-token1', app='testapp1').content_object, testuser)
        self.assertEquals(GenericTokenWithMetadata.objects.count(), 1)
        self.assertEquals(GenericTokenWithMetadata.objects.unsafe_pop(
            token='test-token2', app='testapp2').content_object, testuser)
        self.assertEquals(GenericTokenWithMetadata.objects.count(), 0)

    def test_filter_not_expired(self):
        unexpired_generic_token_with_metadata = self._create_generic_token_with_metadata(
            user=create_user('testuser1'), token='test-token1',
            expiration_datetime=datetime(2015, 1, 1, 14, 30))
        self._create_generic_token_with_metadata(
            user=create_user('testuser2'), token='test-token2',
            expiration_datetime=datetime(2015, 1, 1, 13, 30))

        with mock.patch('django_cradmin.apps.cradmin_generic_token_with_metadata.models._get_current_datetime',
                        lambda: datetime(2015, 1, 1, 14)):
            self.assertEquals(GenericTokenWithMetadata.objects.filter_not_expired().count(), 1)
            self.assertEquals(GenericTokenWithMetadata.objects.filter_not_expired().first(),
                              unexpired_generic_token_with_metadata)

    def test_filter_not_expired_none(self):
        expired_none_generic_token_with_metadata = self._create_generic_token_with_metadata(
            user=create_user('testuser1'), token='test-token1',
            expiration_datetime=None)
        self._create_generic_token_with_metadata(
            user=create_user('testuser2'), token='test-token2',
            expiration_datetime=datetime(2015, 1, 1, 13, 30))

        with mock.patch('django_cradmin.apps.cradmin_generic_token_with_metadata.models._get_current_datetime',
                        lambda: datetime(2015, 1, 1, 14)):
            self.assertEquals(GenericTokenWithMetadata.objects.filter_not_expired().count(), 1)
            self.assertEquals(GenericTokenWithMetadata.objects.filter_not_expired().first(),
                              expired_none_generic_token_with_metadata)

    def test_filter_has_expired(self):
        self._create_generic_token_with_metadata(
            user=create_user('testuser1'),
            app='testapp', token='test-token1',
            expiration_datetime=datetime(2015, 1, 1, 14, 30))
        self._create_generic_token_with_metadata(
            user=create_user('testuser2'),
            app='testapp', token='test-token2',
            expiration_datetime=None)
        expired_generic_token_with_metadata = self._create_generic_token_with_metadata(
            user=create_user('testuser3'),
            app='testapp', token='test-token3',
            expiration_datetime=datetime(2015, 1, 1, 13, 30))

        with mock.patch('django_cradmin.apps.cradmin_generic_token_with_metadata.models._get_current_datetime',
                        lambda: datetime(2015, 1, 1, 14)):
            self.assertEquals(GenericTokenWithMetadata.objects.filter_has_expired().count(), 1)
            self.assertEquals(GenericTokenWithMetadata.objects.filter_has_expired().first(),
                              expired_generic_token_with_metadata)

    def test_pop_not_expired(self):
        testuser = create_user('testuser')
        self._create_generic_token_with_metadata(
            user=testuser, token='test-token',
            expiration_datetime=datetime(2015, 1, 1, 14, 30))

        with mock.patch('django_cradmin.apps.cradmin_generic_token_with_metadata.models._get_current_datetime',
                        lambda: datetime(2015, 1, 1, 14)):
            self.assertEquals(GenericTokenWithMetadata.objects.pop(app='testapp', token='test-token').content_object,
                              testuser)

    def test_pop_expired(self):
        self._create_generic_token_with_metadata(
            user=create_user('testuser'),
            app='testapp', token='test-token',
            expiration_datetime=datetime(2015, 1, 1, 13, 30))

        with mock.patch('django_cradmin.apps.cradmin_generic_token_with_metadata.models._get_current_datetime',
                        lambda: datetime(2015, 1, 1, 14)):
            with self.assertRaises(GenericTokenWithMetadata.DoesNotExist):
                GenericTokenWithMetadata.objects.pop(app='testapp', token='test-token')

    def test_delete_expired(self):
        unexpired_generic_token_with_metadata = self._create_generic_token_with_metadata(
            user=create_user('testuser1'), token='test-token1',
            expiration_datetime=datetime(2015, 1, 1, 14, 30))
        self._create_generic_token_with_metadata(
            user=create_user('testuser2'), token='test-token2',
            expiration_datetime=datetime(2015, 1, 1, 13, 30))

        self.assertEquals(GenericTokenWithMetadata.objects.count(), 2)
        with mock.patch('django_cradmin.apps.cradmin_generic_token_with_metadata.models._get_current_datetime',
                        lambda: datetime(2015, 1, 1, 14)):
            GenericTokenWithMetadata.objects.delete_expired()
        self.assertEquals(GenericTokenWithMetadata.objects.count(), 1)
        self.assertEquals(GenericTokenWithMetadata.objects.first(),
                          unexpired_generic_token_with_metadata)

    def test_delete_expired_management_command(self):
        unexpired_generic_token_with_metadata = self._create_generic_token_with_metadata(
            user=create_user('testuser1'), token='test-token1',
            expiration_datetime=datetime(2015, 1, 1, 14, 30))
        self._create_generic_token_with_metadata(
            user=create_user('testuser2'), token='test-token2',
            expiration_datetime=datetime(2015, 1, 1, 13, 30))

        self.assertEquals(GenericTokenWithMetadata.objects.count(), 2)
        with mock.patch('django_cradmin.apps.cradmin_generic_token_with_metadata.models._get_current_datetime',
                        lambda: datetime(2015, 1, 1, 14)):
            call_command('cradmin_generic_token_with_metadata_delete_expired')
        self.assertEquals(GenericTokenWithMetadata.objects.count(), 1)
        self.assertEquals(GenericTokenWithMetadata.objects.first(),
                          unexpired_generic_token_with_metadata)

    def test_is_expired(self):
        unexpired_generic_token_with_metadata = self._create_generic_token_with_metadata(
            user=create_user('testuser1'), token='test-token1',
            expiration_datetime=datetime(2015, 1, 1, 14, 30))
        expired_generic_token_with_metadata = self._create_generic_token_with_metadata(
            user=create_user('testuser2'), token='test-token2',
            expiration_datetime=datetime(2015, 1, 1, 13, 30))

        with mock.patch('django_cradmin.apps.cradmin_generic_token_with_metadata.models._get_current_datetime',
                        lambda: datetime(2015, 1, 1, 14)):
            self.assertFalse(unexpired_generic_token_with_metadata.is_expired())
            self.assertTrue(expired_generic_token_with_metadata.is_expired())

    def test_is_expired_expiration_datetime_none(self):
        tokenobject = self._create_generic_token_with_metadata(
            user=create_user('testuser2'), token='test-token2',
            expiration_datetime=None)
        self.assertFalse(tokenobject.is_expired())



from django.conf import settings
from django.core.mail import send_mail
from django.template.loader import render_to_string
import html2text


def convert_html_to_plaintext(html):
    """
    Convert the given ``html`` to plain text.
    """
    return html2text.html2text(html)


class AbstractEmail(object):
    """
    Abstract class for sending email.

    Examples:

        Simple example::

            from django_cradmin.apps.cradmin_email import emailutils

            class SimpleEmail(emailutils.AbstractEmail):
                subject_template = 'myapp/subject.django.txt'
                html_message_template = 'myapp/html_message.django.html'

            SimpleEmail(recipient='test@example.com').send()

        Specify a sender::

            SimpleEmail(recipient='test@example.com', from_email='admin@example.com').send()

        Build a common abstract class that defaults to something other
        than ``settings.DEFAULT_FROM_EMAIL``::

            class AbstractAccountEmail(emailutils.AbstractEmail):
                def get_default_from_email(self):
                    return 'account@example.com'

        Specify a plaintext template instead of converting the HTML email
        to plaintext::

            class WithPlaintextEmail(emailutils.AbstractEmail):
                subject_template = 'myapp/subject.django.txt'
                html_message_template = 'myapp/html_message.django.html'
                plaintext_message_template = 'myapp/plaintext_message.django.txt'

        Set subject via a translation string instead of a template
        (you can do this for the message as well)::

            class StringSubjectEmail(emailutils.AbstractEmail):
                html_message_template = 'myapp/html_message.django.html'

                def render_subject(self):
                    return _('My subject')

        Provide context data for the templates::

            SimpleEmail(recipient='test@example.com', extra_context_data={
                'name': 'Peter Pan',
            }).send()
    """

    #: The Django template for the subject. You must set this in subclasses,
    #: or override :meth:`.get_subject_template` or :meth:`.render_subject`.
    subject_template = None

    #: The Django template for the HTML message. You must set this in subclasses,
    #: or override :meth:`.get_html_message_template` or :meth:`.render_html_message`.
    html_message_template = None

    #: The Django template for the Plain text message.
    #: If this or :meth:`.get_plaintext_message_template` or :meth:`.render_subject`
    #: is not set in subclasses, we autoconvert the HTML message to plain text.
    plaintext_message_template = None

    #: Fallback value of the DJANGO_CRADMIN_EMAIL_DEFAULT_CONTEXT_DATA setting
    #: is not set.
    DEFAULT_CONTEXT_DATA = {
        'body_style': 'background-color: #fff;',
        'common_td_style': 'font-family: Arial, sans-serif; '
                           'font-size: 16px; '
                           'line-height: 1.42857143; '
                           'margin: 0; '
                           'letter-spacing: 0.5px; ',
        'header_td_style': '',
        'logo_style': '',
        'contents_td_style': 'padding-top: 20px; padding-bottom: 28px;',
        'footer_td_style': 'border-top: 3px solid #eee; '
                           'font-size: 13px; '
                           'color: #777; '
                           'text-align: center; '
                           'padding-top: 20px; '
                           'padding-bottom: 20px; ',
        'footer_link_style': 'color: #777; text-decoration: underline;',
        'link_style': 'color: #377CA8; text-decoration: underline;',
        'secondary_button_link_style': 'font-size: 16px; '
                                       'font-family: Arial, sans-serif; '
                                       'color: #fff; '
                                       'text-decoration: none; '
                                       'font-weight: bold; '
                                       'text-transform: uppercase; '
                                       'letter-spacing: 1px; '
                                       'background-color: #999999; '
                                       'border-top: 10px solid #999999; '
                                       'border-bottom: 10px solid #999999; '
                                       'border-right: 16px solid #999999; '
                                       'border-left: 16px solid #999999; '
                                       'border-radius: 3px; '
                                       '-webkit-border-radius: 3px; '
                                       '-moz-border-radius: 3px; '
                                       'display: inline-block;',
        'primary_button_link_style': 'font-size: 16px; '
                                     'font-family: Arial, sans-serif; '
                                     'color: #fff; '
                                     'text-decoration: none; '
                                     'font-weight: bold; '
                                     'text-transform: uppercase; '
                                     'letter-spacing: 1px; '
                                     'background-color: #377CA8; '
                                     'border-top: 10px solid #377CA8; '
                                     'border-bottom: 10px solid #377CA8; '
                                     'border-right: 16px solid #377CA8; '
                                     'border-left: 16px solid #377CA8; '
                                     'border-radius: 3px; '
                                     '-webkit-border-radius: 3px; '
                                     '-moz-border-radius: 3px; '
                                     'display: inline-block;',
    }

    def get_subject_template(self):
        """
        Alternative way to specify :obj:`.subject_template`.
        This takes presedence over :obj:`.subject_template`.
        """
        if self.subject_template:
            return self.subject_template
        else:
            raise NotImplementedError('You must override subject_template or get_subject_template')

    def get_html_message_template(self):
        """
        Alternative way to specify :obj:`.html_message_template`.
        This takes presedence over :obj:`.html_message_template`.
        """
        if self.html_message_template:
            return self.html_message_template
        else:
            raise NotImplementedError('You must override html_message_template or get_html_message_template')

    def get_plaintext_message_template(self):
        """
        Alternative way to specify :obj:`.plaintext_message_template`.
        This takes presedence over :obj:`.plaintext_message_template`.
        """
        return self.plaintext_message_template

    def get_subject_prefix(self):
        """
        Get the prefix to use for the subject.

        Defaults to ``settings.DJANGO_CRADMIN_EMAIL_SUBJECT_PREFIX``, falling
        back to empty string.

        If your privide a prefix, you should most likely include an empty
        space at the end of it.
        """
        if hasattr(settings, 'DJANGO_CRADMIN_EMAIL_SUBJECT_PREFIX'):
            return settings.DJANGO_CRADMIN_EMAIL_SUBJECT_PREFIX
        else:
            return ''

    def render_subject(self):
        """
        Render the subject. You can override this if you want to
        adjust template rendering or avoid using a template.
        """
        subject = render_to_string(self.get_subject_template(), self.get_context_data()).strip()
        return '{}{}'.format(self.get_subject_prefix(), subject)

    def render_html_message(self):
        """
        Render the html message. You can override this if you want to
        adjust template rendering or avoid using a template.
        """
        return render_to_string(self.get_html_message_template(), self.get_context_data())

    def __get_rendered_html_message(self):
        """
        We use this internally to avoid rendering the html message
        twice for the default case where the plaintext message is
        created from the HTML message.
        """
        if not hasattr(self, '_rendered_html_message'):
            self._rendered_html_message = self.render_html_message().strip()
        return self._rendered_html_message

    def render_plaintext_message(self):
        """
        Render the plaintext message. You can override this if you want to
        adjust template rendering or avoid using a template.

        If no :obj:`.plaintext_message_template` or :meth:`.get_plaintext_message_template`
        is specified, we convert the HTML message to plaintext using
        :func:`.convert_html_to_plaintext`.
        """
        template_name = self.get_plaintext_message_template()
        if template_name:
            return render_to_string(template_name, self.get_context_data()).strip()
        else:
            return convert_html_to_plaintext(self.__get_rendered_html_message())

    def get_default_context_data(self):
        """
        Get the default context data.

        You should normally not override this, but rather use it if you override
        :meth:`.get_context_data` and calling ``super().get_context_data()``
        is not a suitable solution.
        """
        context_data = {
            'from_email': self.from_email,
            'DJANGO_CRADMIN_SITENAME': getattr(settings, 'DJANGO_CRADMIN_SITENAME', ''),
            'DJANGO_CRADMIN_EMAIL_LOGO_HTML': getattr(settings, 'DJANGO_CRADMIN_EMAIL_LOGO_HTML', '')
        }
        if hasattr(settings, 'DJANGO_CRADMIN_EMAIL_DEFAULT_CONTEXT_DATA'):
            context_data.update(settings.DJANGO_CRADMIN_EMAIL_DEFAULT_CONTEXT_DATA)
        else:
            context_data.update(self.DEFAULT_CONTEXT_DATA)
        return context_data

    def get_context_data(self):
        """
        Get the template context data sent to :obj:`.subject_template`,
        :obj:`.html_message_template` and :obj:`.plaintext_message_template`.

        By default this returns:

        - ``from_email``.
        - ``DJANGO_CRADMIN_SITENAME`` (if set as a Django setting).
        - anything you send as the ``extra_context_data`` argument to the constructor.
        - Anything in the ``DJANGO_CRADMIN_EMAIL_DEFAULT_CONTEXT_DATA`` setting.
        """
        context_data = self.get_default_context_data()
        if self.extra_context_data:
            context_data.update(self.extra_context_data)
        return context_data

    def get_from_email(self):
        """
        Get the email address of the sender. Override this
        if you want to provide the ``from_email`` as a method
        instead of argument to the constructor.
        """
        if self.from_email:
            return self.from_email
        else:
            raise NotImplementedError('You must send from_email as argument to __init__, or override get_from_email().')

    def get_recipient_list(self):
        """
        Get the list of recipients. Override this
        if you want to provide the ``recipient_list`` as a method
        instead of argument to the constructor.
        """
        if self.recipient_list:
            return self.recipient_list
        else:
            raise NotImplementedError('You must send recipient_list or recipient as argument to __init__, '
                                      'or override get_recipient_list().')

    def get_send_mail_kwargs(self):
        """
        Get a dict with kwargs for :meth:`django.core.mail.send_mail`.

        We default to building this from the various methods on this class,
        but you can override this to adjust the kwargs.
        """
        return {
            'subject': self.render_subject(),
            'message': self.render_plaintext_message(),
            'html_message': self.__get_rendered_html_message(),
            'from_email': self.get_from_email(),
            'recipient_list': self.get_recipient_list()
        }

    def send(self):
        """
        Send the email.
        """
        send_mail(**self.get_send_mail_kwargs())

    def get_default_from_email(self):
        """
        Get the fallback value for ``from_email``. Defaults to
        ``settings.DEFAULT_FROM_EMAIL``, but you can override this
        to provide a common superclass that defaults to sending
        from a different address.
        """
        return settings.DEFAULT_FROM_EMAIL

    def __init__(self, recipient=None, recipient_list=None, extra_context_data=None, from_email=None):
        """
        Parameters:
            recipient (str): The recipient of the email.
            recipient_list (list): The recipients of the email. Use this instead of ``recipient``
                if you want to send the message to multiple people.
            extra_context_data (dict): An optional dict with extra context data for the templates.
            from_email: The email address we send the message from.
                Defaults to :meth:`.get_default_from_email`.
        """
        if recipient and recipient_list:
            raise ValueError('You can only specify one of recipient or recipient_list.')
        if recipient:
            self.recipient_list = [recipient]
        else:
            self.recipient_list = recipient_list

        if from_email:
            self.from_email = from_email
        else:
            self.from_email = self.get_default_from_email()

        self.extra_context_data = extra_context_data



from __future__ import unicode_literals
from builtins import object
import collections


class NoCrAdminInstanceFound(Exception):
    """
    Raised by :meth:`.CrAdminInstanceRegistry.get_current_instance`
    when no instance is found.
    """


class CrAdminInstanceRegistry(object):
    """
    We want the _current_ CrAdmin instance to be available
    without having to add it to all the views within the
    instance (we want to add it in a middleware and in a template context).

    At the same time, we want to be able to have multiple instances of CrAdmin
    running on the same site (imagine one site for employees and one for
    customers with a completely different look and feel).

    The solution is simple: If we have only a single instance, we do not
    need to do anything except adding it to this registry. If we have
    multiple instances, each of them must implement
    :meth:`django_cradmin.crinstance.BaseCrAdminInstance.matches_urlpath`.

    Example::

        from django_cradmin.crinstance import BaseCrAdminInstance
        from django_cradmin.registry import cradmin_instance_registry

        class MyCrAadminInstance(BaseCrAdminInstance):
            # ...

        cradmin_instance_registry.add(MyCrAadminInstance)
    """

    def __init__(self):
        self._registry = collections.OrderedDict()

    def get_instanceclass_by_id(self, id):
        """
        Get a CrAdmin instance class by ID (by :obj:`django_cradmin.crinstance.BaseCrAdminInstance.id`).
        """
        return self._registry[id]

    def get_instance_by_id(self, id, request):
        """
        Shortcut for ``get_instanceclass_by_id(id)(request)``.
        """
        return self.get_instanceclass_by_id(id)(request)

    def get_current_instance(self, request):
        """
        Get the current CrAdmin instance.

        Params:
            request (HttpRequest): The http request to the instance for.

        Returns:
            BaseCrAdminInstance: The CrAdmin instance for the given request.

        Raises:
            NoCrAdminInstanceFound: If no instance matching ``request.path`` is found.
        """
        if len(self._registry) == 1:
            return list(self._registry.values())[0](request)
        else:
            for instanceclass in list(self._registry.values()):
                if instanceclass.matches_urlpath(request.path):
                    return instanceclass(request)
            raise NoCrAdminInstanceFound('No CrAdminInstance matching {path} found'.format(
                path=request.path))

    def add(self, cradmin_instance_class):
        """
        Add a CrAdmin instance to the registry.

        Parameters:
            cradmin_instance_class: A subclass of :class:`django_cradmin.crinstance.BaseCrAdminInstance`.
        """
        if cradmin_instance_class.id not in self._registry:
            self._registry[cradmin_instance_class.id] = cradmin_instance_class


cradmin_instance_registry = CrAdminInstanceRegistry()



from __future__ import unicode_literals
from django.contrib import admin

from django_cradmin.apps.cradmin_imagearchive.models import ArchiveImage


class ArchiveImageAdmin(admin.ModelAdmin):
    list_display = (
        'name',
        'description',
        'role',
    )

    search_fields = [
        'name',
        'description',
    ]

    readonly_fields = [
        'image_width',
        'image_height',
        'file_extension',
    ]

    def get_queryset(self, request):
        if request.user.is_superuser:
            return super(ArchiveImageAdmin, self).get_queryset(request)
        else:
            return ArchiveImage.objects.none()

admin.site.register(ArchiveImage, ArchiveImageAdmin)



from __future__ import unicode_literals
from builtins import str
from collections import OrderedDict
import textwrap
from django import template
from django.template.base import parse_bits

register = template.Library()


class WrapTextNode(template.Node):
    def __init__(self, nodelist, max_line_length, indentspaces=0):
        self.nodelist = nodelist
        self.max_line_length = max_line_length
        self.indentspaces = indentspaces
        # self.max_line_length = 10
        # self.indentspaces = 4
        # self.max_line_length = int(max_line_length[0])
        # self.indentspaces = int(indentspaces['indentspaces'])

    def render(self, context):
        output = self.nodelist.render(context)
        indent = ' ' * self.indentspaces
        wrapper = textwrap.TextWrapper(
            width=self.max_line_length,
            break_long_words=False,
            initial_indent=indent,
            subsequent_indent=indent)
        return wrapper.fill(output)


def parse_templatetag_arguments(parser, token, specification):
    split = token.split_contents()
    tag_name = split[0]
    bits = split[1:]

    params = list(specification.keys())

    args, kwargs = parse_bits(parser, bits,
                              params=params,
                              varargs=None, varkw=None,
                              defaults=[],
                              name='unused',
                              takes_context=False)
    for key in kwargs:
        kwargs[key] = str(kwargs[key])

    all_args_dict = {}
    for index, param in enumerate(params):
        try:
            value = args[index]
        except IndexError:
            try:
                defaultvalue = specification[param]['default']
            except KeyError:
                raise template.TemplateSyntaxError("{!r} tags '{}'-argument is required".format(
                    tag_name, param))
            else:
                value = kwargs.get(param, defaultvalue)
        typecast = specification[param]['typecast']
        try:
            all_args_dict[param] = typecast(str(value))
        except ValueError:
            raise template.TemplateSyntaxError("{!r} tags '{}'-argument must be an {}".format(
                tag_name, param, typecast))
    return tag_name, all_args_dict


@register.tag
def cradmin_wrap_text(parser, token):
    tag_name, kwargs = parse_templatetag_arguments(
        parser, token,
        specification=OrderedDict([
            ('max_line_length', {'typecast': int}),
            ('indentspaces', {'default': 0, 'typecast': int}),
        ]))

    max_line_length = kwargs['max_line_length']
    indentspaces = kwargs['indentspaces']
    nodelist = parser.parse(('end_cradmin_wrap_text',))
    parser.delete_first_token()
    return WrapTextNode(nodelist, max_line_length, indentspaces=indentspaces)



from __future__ import unicode_literals
from django.contrib import admin
from django_cradmin.apps.cradmin_temporaryfileuploadstore.models import TemporaryFileCollection, TemporaryFile


class TemporaryFileInline(admin.StackedInline):
    fields = ['filename', 'file']
    readonly_fields = ['filename', 'file']
    model = TemporaryFile
    extra = 0


class TemporaryFileCollectionAdmin(admin.ModelAdmin):
    inlines = [TemporaryFileInline]
    list_display = (
        'id',
        'user',
        'created_datetime',
        'minutes_to_live',
    )
    readonly_fields = [
        'user',
        'accept',
        'created_datetime',
        'unique_filenames',
        'max_filename_length']
    search_fields = [
        'id',
        'user__id',
        'user__email',
    ]
    date_hierarchy = 'created_datetime'

    def get_queryset(self, request):
        if request.user.is_superuser:
            return super(TemporaryFileCollectionAdmin, self).get_queryset(request)
        else:
            return TemporaryFile.objects.none()

admin.site.register(TemporaryFileCollection, TemporaryFileCollectionAdmin)



from __future__ import unicode_literals
from django.core.management.base import NoArgsCommand
from django_cradmin.apps.cradmin_generic_token_with_metadata.models import GenericTokenWithMetadata


class Command(NoArgsCommand):
    help = 'Delete all expired GenericTokenWithMetadata objects from the database.'

    def handle_noargs(self, **options):
        GenericTokenWithMetadata.objects.delete_expired()



from __future__ import unicode_literals
from builtins import range
from builtins import object
from django.core.exceptions import ValidationError
from django.db import models


class SortableManagerBase(models.Manager):
    """
    Manager for :class:`.SortableBase`.
    """
    parent_attribute = None

    def _get_filtered_by_parentobject_queryset(self, parentobject):
        filter_kwargs = {self.parent_attribute: parentobject}
        return self.get_queryset().filter(**filter_kwargs).order_by('sort_index')

    def _get_siblings_queryset(self, item):
        """
        Get the queryset for all the items with the same parent as the given
        item.
        """
        parentobject = getattr(item, self.parent_attribute)
        return self._get_filtered_by_parentobject_queryset(parentobject)

    def _get_last_sortindex_within_parentobject(self, item):
        lastitem = self._get_siblings_queryset(item).last()
        if lastitem:
            return lastitem.sort_index + 1
        else:
            return 0

    def set_newitem_sort_index_to_last(self, item):
        """
        Sets ``item.sort_index`` to the sort_index of the last item
        in the parent + 1. Does not save.

        ONLY USE THIS FOR NEWLY CREATED ITEMS.
        """
        if item.sort_index is not None or item.pk is not None:
            raise ValueError('set_newitem_sort_index_to_last should only be used '
                             'when creating new items. sort_index or pk is something '
                             'other than None, which indicates that this is not a new '
                             'item - thus the ValueError.')
        item.sort_index = self._get_last_sortindex_within_parentobject(item)

    def __increase_sort_index_in_range(self, queryset, items, from_index, to_index, distance=1):
        """
        Used by sort_before to increase the sort_index of all items in the given range
        """
        increase_index_for_items = [itm.id for itm in items[from_index:to_index]]
        queryset.filter(id__in=increase_index_for_items).update(sort_index=models.F('sort_index') + distance)

    def __decrease_sort_index_in_range(self, queryset, items, from_index, to_index, distance=1):
        """
        Used by sort_before to decrease the sort_index of all items in the given range
        """
        decrease_index_for_items = [itm.id for itm in items[from_index:to_index]]
        queryset.filter(id__in=decrease_index_for_items).update(sort_index=models.F('sort_index') - distance)

    def sort_before(self, item, sort_before_id):
        """
        Sort a given item before the item with id `sort_before_id`,
        or last if `sort_before_id` is ``None``.

        Fetches all items in the same container, and makes changes in the ordering.
        Only the required updates are made.
        """
        queryset = self._get_siblings_queryset(item)
        items = queryset.all()
        # Get the current position for the item to sort before
        sort_before_index = None
        original_item_index = None

        for index in range(0, len(items)):
            cur_item = items[index]
            if index < cur_item.sort_index:
                # Found gap, need to move rest of list <size-of-gap> step(s) down
                self.__decrease_sort_index_in_range(queryset, items, index, len(items), cur_item.sort_index-index)
                items = queryset.all()
            if index > cur_item.sort_index:
                # Found duplicate sort_index, need to move rest of list one step up
                self.__increase_sort_index_in_range(queryset, items, index, len(items))
                items = queryset.all()

            if item.id == cur_item.id:
                original_item_index = cur_item.sort_index

            elif sort_before_id is not None and cur_item.id == sort_before_id:
                sort_before_index = cur_item.sort_index

        item.sort_index = None
        if sort_before_index is None:
            # Place last.
            if original_item_index is not None:
                # fill gap left when moving item to end
                self.__decrease_sort_index_in_range(queryset, items, original_item_index+1, len(items))
                item.sort_index = len(items)-1
            else:
                item.sort_index = len(items)
            item.save()
        else:
            # Place somewhere not last
            if original_item_index is None:
                # new item, move rest of list one step up, and place the new item
                self.__increase_sort_index_in_range(queryset, items, sort_before_index, len(items))
                item.sort_index = sort_before_index
                item.save()
            elif original_item_index < sort_before_index:
                # Move up, and fill gap left behind by moving items in the gap down
                self.__decrease_sort_index_in_range(queryset, items, original_item_index+1, sort_before_index)
                item.sort_index = sort_before_index-1
                item.save()
            elif original_item_index > sort_before_index:
                # Move down, and fill/create gap by moving other objects up
                self.__increase_sort_index_in_range(queryset, items, sort_before_index, original_item_index)
                item.sort_index = sort_before_index
                item.save()

    def sort_last(self, item):
        """
        Just a shortcut for::

            self.sort_before(item, sort_before_id=None)
        """
        self.sort_before(item, sort_before_id=None)


def validate_sort_index(value):
    if value < 0:
        raise ValidationError(u'Sort index must be 0 or higher.')


class SortableBase(models.Model):
    """
    Used with :class:`.SortableManagerBase` to make models sortable.
    """

    #: Sort index - ``0`` or higher.
    sort_index = models.PositiveIntegerField(
        blank=True, null=True,
        default=None,
        verbose_name='index',
        validators=[validate_sort_index]
    )

    class Meta(object):
        abstract = True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django_cradmin.apps.cradmin_imagearchive.models


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ArchiveImage',
            fields=[
                ('id', models.AutoField(primary_key=True, auto_created=True, verbose_name='ID', serialize=False)),
                ('role_object_id', models.PositiveIntegerField()),
                ('image', models.ImageField(upload_to=django_cradmin.apps.cradmin_imagearchive.models.archiveimage_upload_to, null=True, help_text='Select an image to add to the archive.', verbose_name='image', height_field='image_height', max_length=255, width_field='image_width')),
                ('image_height', models.IntegerField(null=True, blank=True, editable=False)),
                ('image_width', models.IntegerField(null=True, blank=True, editable=False)),
                ('file_extension', models.CharField(max_length=255, editable=False)),
                ('name', models.CharField(max_length=255, blank=True, help_text='Give the image a name (optional). If you leave this blank, the name of the uploaded image file is used.', verbose_name='name')),
                ('description', models.TextField(default='', blank=True, help_text='An optional description of the image. Think if this as a description of the image for visually impaired users. This means that you should describe the information carried in the image (if any).', verbose_name='description')),
                ('created_datetime', models.DateTimeField(auto_now_add=True)),
                ('role_content_type', models.ForeignKey(help_text='The role owning this image.', verbose_name='role', to='contenttypes.ContentType')),
            ],
            options={
                'verbose_name_plural': 'archive images',
                'verbose_name': 'archive image',
            },
            bases=(models.Model,),
        ),
    ]



from __future__ import unicode_literals
from django.contrib import messages
from django.http import HttpResponseRedirect
from django.template.loader import render_to_string
from django.views.generic import TemplateView

from django_cradmin.apps.cradmin_generic_token_with_metadata.models import GenericTokenWithMetadata, \
    GenericTokenExpiredError


class ActivateAccountView(TemplateView):
    template_name = 'cradmin_activate_account/activate.django.html'
    appname = 'cradmin_activate_account'

    #: The template used to render the success message.
    #: Default value for :obj:`~.ActivateAccountView.get_success_message_template`
    success_message_template = 'cradmin_activate_account/messages/success.django.html'

    def get(self, *args, **kwargs):
        self.token_does_not_exist = False
        self.token_expired = False
        try:
            token = GenericTokenWithMetadata.objects.get_and_validate(
                token=self.kwargs['token'], app=self.appname)
        except GenericTokenWithMetadata.DoesNotExist:
            self.token_does_not_exist = True
        except GenericTokenExpiredError:
            self.token_expired = True
        else:
            return self.token_is_valid(token)
        return super(ActivateAccountView, self).get(*args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(ActivateAccountView, self).get_context_data(**kwargs)
        context['token_does_not_exist'] = self.token_does_not_exist
        context['token_expired'] = self.token_expired
        return context

    def get_success_message_template(self):
        """
        Get the template used to render the success message.
        Defaults to :obj:`~.ActivateAccountView.success_message_template`.
        """
        return self.success_message_template

    def get_success_message(self, user):
        """
        Get the success message added by :meth:`.add_success_message`.

        Defaults to rendering the :meth:`.get_success_message_template`
        template.
        """
        return render_to_string(self.get_success_message_template(), {
            'user': user
        }).strip()

    def add_success_message(self, user):
        """
        Add success message.

        Defaults to adding the value of :meth:`~.ActivateAccountView.get_success_message`
        as a django messages framework success message.
        """
        messages.success(self.request, self.get_success_message(user))

    def activate_user(self, user):
        """
        Activate the user.

        You can override this to provide custom user activation code,
        but you will most likely want to override the ``activate_user()``
        method of your User model if you have a custom user model.
        """
        if hasattr(user, 'activate_user'):
            user.activate_user()
        else:
            user.is_active = True
        user.save()

    def token_is_valid(self, token):
        user = token.content_object
        next_url = token.metadata['next_url']
        self.activate_user(user)
        self.add_success_message(user)
        return HttpResponseRedirect(next_url)



from __future__ import unicode_literals
from django.conf.urls import url, patterns
from django_cradmin.apps.cradmin_resetpassword.views.begin import BeginPasswordResetView
from django_cradmin.apps.cradmin_resetpassword.views.email_sent import EmailSentView
from django_cradmin.apps.cradmin_resetpassword.views.reset import ResetPasswordView

urlpatterns = patterns(
    '',
    url(r'^begin',
        BeginPasswordResetView.as_view(),
        name="cradmin-resetpassword-begin"),
    url(r'^email-sent',
        EmailSentView.as_view(),
        name="cradmin-resetpassword-email-sent"),
    url(r'^reset/(?P<token>.+)',
        ResetPasswordView.as_view(),
        name="cradmin-resetpassword-reset"),
)



from __future__ import unicode_literals
from django.conf.urls import url, patterns

from django_cradmin.apps.cradmin_register_account.views.begin import BeginRegisterAccountView
from django_cradmin.apps.cradmin_register_account.views.email_sent import EmailSentView


urlpatterns = patterns(
    '',
    url(r'^begin',
        BeginRegisterAccountView.as_view(),
        name="cradmin-register-account-begin"),
    url(r'^email-sent',
        EmailSentView.as_view(),
        name="cradmin-register-account-email-sent"),
)



from __future__ import unicode_literals
import json
import mimetypes
from django.conf import settings
from django.core.exceptions import ValidationError
from django.http import HttpResponse
from django.views.generic import FormView
from django.utils.translation import ugettext_lazy as _

from django import forms
from multiupload.fields import MultiFileField
from django_cradmin.apps.cradmin_temporaryfileuploadstore.models import TemporaryFileCollection, TemporaryFile, \
    html_input_accept_match, make_unique_filename


class FileUploadForm(forms.Form):
    file = MultiFileField(
        max_file_size=1000000 * getattr(settings, 'CRADMIN_TEMPORARYFILEUPLOADSTORE_MAX_FILE_SIZE_MB', 1000))
    collectionid = forms.IntegerField(
        required=False)
    minutes_to_live = forms.IntegerField(
        min_value=1,
        required=False)
    mode = forms.ChoiceField(
        required=False,
        choices=[
            (None, 'Multifile'),
            ('singlefile', 'Single file')
        ]
    )
    accept = forms.CharField(required=False)
    max_filename_length = forms.IntegerField(min_value=0, required=False)
    unique_filenames = forms.BooleanField(required=False)


class FileDeleteForm(forms.Form):
    collectionid = forms.IntegerField(required=True)
    temporaryfileid = forms.IntegerField(required=True)


class UploadTemporaryFilesView(FormView):
    form_class = FileUploadForm
    http_method_names = ['post', 'delete']

    def create_collection(self, minutes_to_live, accept, max_filename_length, unique_filenames):
        collection = TemporaryFileCollection(
            user=self.request.user)
        if minutes_to_live is not None:
            collection.minutes_to_live = minutes_to_live
        if accept is not None:
            collection.accept = accept
        if max_filename_length:
            collection.max_filename_length = max_filename_length
        if unique_filenames:
            collection.unique_filenames = unique_filenames
        collection.full_clean()
        collection.save()
        return collection

    def get_existing_collection(self, collectionid):
        return TemporaryFileCollection.objects.filter(user=self.request.user).get(id=collectionid)

    def create_or_get_collection_id(self, collectionid, minutes_to_live, accept,
                                    max_filename_length, unique_filenames):
        if collectionid is None:
            return self.create_collection(
                minutes_to_live=minutes_to_live,
                accept=accept,
                max_filename_length=max_filename_length,
                unique_filenames=unique_filenames)
        else:
            return self.get_existing_collection(collectionid)

    def save_uploaded_file(self, collection, formfile, mode, filename_set):
        if mode == 'singlefile':
            collection.clear_files()

        filename = formfile.name
        if collection.unique_filenames:
            filename = make_unique_filename(
                filename_set=filename_set,
                wanted_filename=filename,
                max_filename_length=collection.max_filename_length)
            filename_set.add(filename)
        temporaryfile = TemporaryFile(collection=collection, filename=filename)
        temporaryfile.file.save(formfile.name, formfile, save=False)
        temporaryfile.full_clean()
        temporaryfile.save()
        return temporaryfile

    def form_invalid(self, form):
        return self.__invalid_form_response(form)

    def everything_valid(self, collection, form):
        uploadedfiles_data = []
        filename_set = None
        if collection.unique_filenames:
            filename_set = collection.get_filename_set()
        for formfile in form.cleaned_data['file']:
            temporaryfile = self.save_uploaded_file(
                collection=collection,
                formfile=formfile,
                mode=form.cleaned_data['mode'],
                filename_set=filename_set)
            uploadedfiles_data.append({
                'id': temporaryfile.id,
                'filename': temporaryfile.filename,
                'mimetype': temporaryfile.mimetype
            })
        return self.json_response(json.dumps({
            'collectionid': collection.id,
            'temporaryfiles': uploadedfiles_data
        }))

    def validate_all_files(self, accept, form):
        if not accept:
            return
        for formfile in form.cleaned_data['file']:
            if not html_input_accept_match(
                    accept=accept,
                    mimetype=mimetypes.guess_type(formfile.name)[0],
                    filename=formfile.name):
                raise ValidationError(
                    _('%(filename)s: Unsupported filetype.') % {'filename': formfile.name},
                    code='unsupported_mimetype')

    def form_valid(self, form):
        collectionid = form.cleaned_data['collectionid']
        minutes_to_live = form.cleaned_data['minutes_to_live']
        accept = form.cleaned_data['accept']
        max_filename_length = form.cleaned_data['max_filename_length']
        unique_filenames = form.cleaned_data['unique_filenames']
        try:
            self.validate_all_files(accept=accept, form=form)
        except ValidationError as e:
            return self.json_response(json.dumps({
                'file': [
                    {
                        'message': e.message,
                        'code': e.code
                    }
                ]
            }), status=400)
        else:
            try:
                collection = self.create_or_get_collection_id(
                    collectionid=collectionid,
                    minutes_to_live=minutes_to_live,
                    accept=accept,
                    max_filename_length=max_filename_length,
                    unique_filenames=unique_filenames)
            except TemporaryFileCollection.DoesNotExist:
                return self.__collection_does_not_exist_response(collectionid)
            else:
                return self.everything_valid(collection=collection, form=form)

    def json_response(self, data, status=200):
        return HttpResponse(
            data,
            content_type='application/json',
            status=status)

    def __collection_does_not_exist_response(self, collectionid):
        return self.json_response(json.dumps({
            'collectionid': [
                {
                    'message': 'Collection with id={} does not exist.'.format(collectionid),
                    'code': 'doesnotexist'
                }
            ]
        }), status=404)

    def __temporaryfile_does_not_exist_response(self, temporaryfileid):
        return self.json_response(json.dumps({
            'temporaryfileid': [
                {
                    'message': 'Temporary file with id={} does not exist.'.format(temporaryfileid),
                    'code': 'doesnotexist'
                }
            ]
        }), status=404)

    def __invalid_form_response(self, form):
        return self.json_response(form.errors.as_json(), status=400)

    def delete(self, request, *args, **kwargs):
        try:
            requestdata = json.loads(self.request.body.decode('utf-8'))
        except ValueError:
            return self.json_response(json.dumps({
                'errormessage': 'Invalid JSON data in the request body.'
            }), status=400)

        form = FileDeleteForm(requestdata)
        if form.is_valid():
            collectionid = form.cleaned_data['collectionid']
            temporaryfileid = form.cleaned_data['temporaryfileid']
            try:
                collection = self.get_existing_collection(collectionid)
            except TemporaryFileCollection.DoesNotExist:
                return self.__collection_does_not_exist_response(collectionid)
            else:
                try:
                    temporaryfile = collection.files.get(id=temporaryfileid)
                except TemporaryFile.DoesNotExist:
                    return self.__temporaryfile_does_not_exist_response(temporaryfileid)
                else:
                    return self.perform_temporaryfile_delete(temporaryfile)
        else:
            return self.__invalid_form_response(form)

    def perform_temporaryfile_delete(self, temporaryfile):
        temporaryfileid = temporaryfile.id
        temporaryfile.delete_object_and_file()
        return self.json_response(json.dumps({
            'collectionid': temporaryfile.collection_id,
            'temporaryfileid': temporaryfileid
        }))
